/* main.c - application indicator to auto-mute Spotify when detecting ads.
   Written in 2014 by Mehmet Kayaalp mkayaalp@cs.binghamton.edu
   
To the extent possible under law, the author(s) have dedicated all copyright
and related and neighboring rights to this software to the public domain
worldwide. This software is distributed without any warranty.
You should have received a copy of the CC0 Public Domain Dedication along with
this software. If not, see 
<http://creativecommons.org/publicdomain/zero/1.0/>.*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <gtk/gtk.h>
#include <libappindicator/app-indicator.h>
#include "xprop.h"
#include "tst.h"
#include "pactl.h"

#define STR(s)                  #s
#define XSTR(s)                 STR(s)
#define LIST_FILE_NAME          "indicator-muteads-list"
#define DEFAULT_LIST_FILE       XSTR(PKGDATADIR) "/" LIST_FILE_NAME
#define STATUS_NOT_PLAYING      "Spotify is not playing"
#define STATUS_NOT_RUNNING      "Spotify is not running"
#define TITLE_DEFAULT           "Spotify"
#define TITLE_OFFSET            10

//modified icon made by Elegant Themes from Flaticon [www.flaticon.com]
#define ICON_DEFAULT            XSTR(PKGDATADIR) "/indicator-muteads.svg"
#define ICON_MUTED              XSTR(PKGDATADIR) "/indicator-muteads-muted.svg"

#define INTERVAL_LONG		30
#define INTERVAL_SHORT		2

enum state_t {
  NOT_RUNNING,
  NOT_PLAYING,
  UNMUTED,
  MUTED
};

static GtkWidget *status_mi;
static GtkWidget *mute_mi;
static GtkWidget *unmute_mi;
static gulong mute_handler;
static gulong unmute_handler;
static AppIndicator *indicator;
struct node* root;
struct node* current_node;
static gboolean file_changed=FALSE;
static int check_interval = INTERVAL_LONG;
static int current_interval = INTERVAL_LONG;
static char* file_name;

static enum state_t state = NOT_RUNNING;

static char* title = NULL;

static void exiting(GtkWidget *widget, gpointer data) {
  gtk_main_quit ();
}

static void update(char* title, gboolean activate, gboolean mute) {
  gtk_menu_item_set_label(GTK_MENU_ITEM(status_mi), title);
  if(state!=MUTED && mute) {
    g_signal_handler_block(mute_mi, mute_handler);
    g_signal_handler_block(unmute_mi, unmute_handler);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mute_mi), TRUE);
    g_signal_handler_unblock(mute_mi, mute_handler);
    g_signal_handler_unblock(unmute_mi, unmute_handler);
    app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ATTENTION);
  }
  else if (state==MUTED && !mute) {
    g_signal_handler_block(mute_mi, mute_handler);
    g_signal_handler_block(unmute_mi, unmute_handler);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(unmute_mi), TRUE);
    g_signal_handler_unblock(mute_mi, mute_handler);
    g_signal_handler_unblock(unmute_mi, unmute_handler);
    app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ACTIVE);
  }
  if((state<=NOT_PLAYING && activate) || (state>=UNMUTED && !activate)) {
    gtk_widget_set_sensitive(GTK_WIDGET(mute_mi), activate);
    gtk_widget_set_sensitive(GTK_WIDGET(unmute_mi), activate);
  }
}

static void refresh(GtkWidget *widget, gpointer data) {
  char* t;
  char* t_actual;
  int retry = 0;
  while(TRUE) {
    if(state==NOT_RUNNING || retry) {
      if(update_window()!=0) {
        if(state!=NOT_RUNNING) {
          update(STATUS_NOT_RUNNING,FALSE,FALSE);
          state=NOT_RUNNING;
          check_interval=INTERVAL_LONG;
        }
        return;
      }
    }
    if(!(t=get_title()) && retry)
      fprintf(stderr,"get title failed");
    else if(!t)
      retry=1;
    else
      break;
  }
  t_actual=&t[TITLE_OFFSET];
  if(!strcmp(t,TITLE_DEFAULT)) {
    if(state!=NOT_PLAYING) {
      update(STATUS_NOT_PLAYING,FALSE,FALSE);
      state=NOT_PLAYING;
      check_interval=INTERVAL_LONG;
    }
  }
  else if(title==NULL || strcmp(t,title)) {
    if(state==NOT_RUNNING || state==NOT_PLAYING)
      update_sink();
    current_node = search(root, t_actual);
    if(current_node) {
      update(t_actual,TRUE,TRUE);
      if(state!=MUTED) {
        set_mute(TRUE);
        state=MUTED;
        check_interval=INTERVAL_SHORT;
      }
    }
    else {
      update(t_actual,TRUE,FALSE);
      if(state!=UNMUTED) {
        set_mute(FALSE);
        state=UNMUTED;
        check_interval=INTERVAL_SHORT;
      }
    }
  }
  free_title(title);
  title=t;
}

static void mute(GtkWidget *widget, gpointer data) {
  if(state==UNMUTED) {
    set_mute(TRUE);
    app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ATTENTION);
    state=MUTED;
    if(current_node)
      unmark_removed(current_node);
    else {
      insert_line(root,&title[TITLE_OFFSET]);
      file_changed=TRUE;
    }
  }
}

static void unmute(GtkWidget *widget, gpointer data) {
  if(state==MUTED) {
    set_mute(FALSE);
    app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ACTIVE);
    state=UNMUTED;
    current_node=search(root, &title[TITLE_OFFSET]);
    if(current_node)
      mark_removed(current_node);
    else
      fprintf(stderr,"unmute(): current_node is NULL\n");
  }
}

static gboolean timed_refresh(gpointer data) {
  refresh(NULL, NULL);
  if(current_interval!=check_interval) {
    g_timeout_add_seconds (check_interval, timed_refresh, NULL);
    current_interval = check_interval;
    return FALSE;
  }
  return TRUE;
}

static void cleanup() {
  if(file_changed)
    write_to_file(root, file_name);
  drain();
}

char* get_list_file_name() {
  struct passwd *pw;
  const char *homedir;
  size_t len;
  char* fullpath;
  int ret;
  pw = getpwuid(getuid());
  if(!pw) {
    perror("getpwuid failed");
    exit(EXIT_FAILURE);
  }
  homedir = pw->pw_dir;
  len = strlen(homedir) + strlen(LIST_FILE_NAME) + 3;
  fullpath = malloc(len);
  if (fullpath == NULL) {
    perror("malloc failed");
    exit(EXIT_FAILURE);
  };
  ret = snprintf(fullpath, len, "%s/.%s", homedir, LIST_FILE_NAME);
  if(ret != len-1) {
    perror("snprintf failed");
    exit(EXIT_FAILURE);
  }
  fprintf(stderr, "get_list_file_name(): filename=%s\n", fullpath);
  return fullpath;
}

int main (int argc, char **argv) {
  GtkWidget *menu;
  GtkWidget *refresh_mi;
  GtkWidget *sep1;
  GtkWidget *sep2;
  GtkWidget *exit_mi;
  GSList *group = NULL;

  
  atexit(cleanup);
  gtk_init(&argc, &argv);
  init_pactl();
  
  file_name = get_list_file_name();
  root = populate_from_file(file_name, DEFAULT_LIST_FILE);
  menu = gtk_menu_new();
  status_mi = gtk_menu_item_new_with_label(STATUS_NOT_RUNNING);
  refresh_mi = gtk_menu_item_new_with_mnemonic("_Refresh");

  sep1 = gtk_separator_menu_item_new();
  mute_mi = gtk_radio_menu_item_new_with_mnemonic(group, "_Mute");
  group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(mute_mi));
  unmute_mi = gtk_radio_menu_item_new_with_mnemonic (group, "_Unmute");
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(unmute_mi), TRUE);

  gtk_widget_set_sensitive(GTK_WIDGET(status_mi), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(mute_mi), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(unmute_mi), FALSE);
  sep2 = gtk_separator_menu_item_new();
  exit_mi = gtk_menu_item_new_with_mnemonic("_Exit");

  gtk_menu_shell_append(GTK_MENU_SHELL(menu), status_mi);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), refresh_mi);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep1);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), mute_mi);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), unmute_mi);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep2);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), exit_mi);

  g_signal_connect(refresh_mi, "activate", G_CALLBACK(refresh), NULL);
  mute_handler = g_signal_connect(mute_mi, "activate", G_CALLBACK(mute), NULL);
  unmute_handler = g_signal_connect(unmute_mi, "activate", G_CALLBACK(unmute), NULL);
  g_signal_connect(exit_mi, "activate", G_CALLBACK(exiting), NULL);

  gtk_widget_show(status_mi);
  gtk_widget_show(refresh_mi);
  gtk_widget_show(sep1);
  gtk_widget_show(mute_mi);
  gtk_widget_show(unmute_mi);
  gtk_widget_show(sep2);
  gtk_widget_show(exit_mi);

  indicator = app_indicator_new(PACKAGE_NAME, ICON_DEFAULT, APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
  app_indicator_set_attention_icon(indicator, ICON_MUTED);
  app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ACTIVE);
  app_indicator_set_menu(indicator, GTK_MENU(menu));

  g_timeout_add_seconds(check_interval, timed_refresh, NULL);
  refresh(NULL, NULL);
  gtk_main();

  return 0;
}
